<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Gettext\\BaseTranslator' => $vendorDir . '/gettext/gettext/src/BaseTranslator.php',
    'Gettext\\Extractors\\Blade' => $vendorDir . '/gettext/gettext/src/Extractors/Blade.php',
    'Gettext\\Extractors\\CsvDictionary' => $vendorDir . '/gettext/gettext/src/Extractors/CsvDictionary.php',
    'Gettext\\Extractors\\Extractor' => $vendorDir . '/gettext/gettext/src/Extractors/Extractor.php',
    'Gettext\\Extractors\\ExtractorInterface' => $vendorDir . '/gettext/gettext/src/Extractors/ExtractorInterface.php',
    'Gettext\\Extractors\\Jed' => $vendorDir . '/gettext/gettext/src/Extractors/Jed.php',
    'Gettext\\Extractors\\JsCode' => $vendorDir . '/gettext/gettext/src/Extractors/JsCode.php',
    'Gettext\\Extractors\\JsonDictionary' => $vendorDir . '/gettext/gettext/src/Extractors/JsonDictionary.php',
    'Gettext\\Extractors\\Mo' => $vendorDir . '/gettext/gettext/src/Extractors/Mo.php',
    'Gettext\\Extractors\\PhpArray' => $vendorDir . '/gettext/gettext/src/Extractors/PhpArray.php',
    'Gettext\\Extractors\\PhpCode' => $vendorDir . '/gettext/gettext/src/Extractors/PhpCode.php',
    'Gettext\\Extractors\\Po' => $vendorDir . '/gettext/gettext/src/Extractors/Po.php',
    'Gettext\\Extractors\\Twig' => $vendorDir . '/gettext/gettext/src/Extractors/Twig.php',
    'Gettext\\Extractors\\YamlDictionary' => $vendorDir . '/gettext/gettext/src/Extractors/YamlDictionary.php',
    'Gettext\\Generators\\CsvDictionary' => $vendorDir . '/gettext/gettext/src/Generators/CsvDictionary.php',
    'Gettext\\Generators\\Generator' => $vendorDir . '/gettext/gettext/src/Generators/Generator.php',
    'Gettext\\Generators\\GeneratorInterface' => $vendorDir . '/gettext/gettext/src/Generators/GeneratorInterface.php',
    'Gettext\\Generators\\Jed' => $vendorDir . '/gettext/gettext/src/Generators/Jed.php',
    'Gettext\\Generators\\JsonDictionary' => $vendorDir . '/gettext/gettext/src/Generators/JsonDictionary.php',
    'Gettext\\Generators\\Mo' => $vendorDir . '/gettext/gettext/src/Generators/Mo.php',
    'Gettext\\Generators\\PhpArray' => $vendorDir . '/gettext/gettext/src/Generators/PhpArray.php',
    'Gettext\\Generators\\Po' => $vendorDir . '/gettext/gettext/src/Generators/Po.php',
    'Gettext\\Generators\\YamlDictionary' => $vendorDir . '/gettext/gettext/src/Generators/YamlDictionary.php',
    'Gettext\\GettextTranslator' => $vendorDir . '/gettext/gettext/src/GettextTranslator.php',
    'Gettext\\Languages\\Category' => $vendorDir . '/gettext/languages/src/Category.php',
    'Gettext\\Languages\\CldrData' => $vendorDir . '/gettext/languages/src/CldrData.php',
    'Gettext\\Languages\\Exporter\\Docs' => $vendorDir . '/gettext/languages/src/Exporter/Docs.php',
    'Gettext\\Languages\\Exporter\\Exporter' => $vendorDir . '/gettext/languages/src/Exporter/Exporter.php',
    'Gettext\\Languages\\Exporter\\Html' => $vendorDir . '/gettext/languages/src/Exporter/Html.php',
    'Gettext\\Languages\\Exporter\\Json' => $vendorDir . '/gettext/languages/src/Exporter/Json.php',
    'Gettext\\Languages\\Exporter\\Php' => $vendorDir . '/gettext/languages/src/Exporter/Php.php',
    'Gettext\\Languages\\Exporter\\Po' => $vendorDir . '/gettext/languages/src/Exporter/Po.php',
    'Gettext\\Languages\\Exporter\\Prettyjson' => $vendorDir . '/gettext/languages/src/Exporter/Prettyjson.php',
    'Gettext\\Languages\\Exporter\\Xml' => $vendorDir . '/gettext/languages/src/Exporter/Xml.php',
    'Gettext\\Languages\\FormulaConverter' => $vendorDir . '/gettext/languages/src/FormulaConverter.php',
    'Gettext\\Languages\\Language' => $vendorDir . '/gettext/languages/src/Language.php',
    'Gettext\\Translation' => $vendorDir . '/gettext/gettext/src/Translation.php',
    'Gettext\\Translations' => $vendorDir . '/gettext/gettext/src/Translations.php',
    'Gettext\\Translator' => $vendorDir . '/gettext/gettext/src/Translator.php',
    'Gettext\\TranslatorInterface' => $vendorDir . '/gettext/gettext/src/TranslatorInterface.php',
    'Gettext\\Utils\\FunctionsScanner' => $vendorDir . '/gettext/gettext/src/Utils/FunctionsScanner.php',
    'Gettext\\Utils\\JsFunctionsScanner' => $vendorDir . '/gettext/gettext/src/Utils/JsFunctionsScanner.php',
    'Gettext\\Utils\\ParsedFunction' => $vendorDir . '/gettext/gettext/src/Utils/ParsedFunction.php',
    'Gettext\\Utils\\PhpFunctionsScanner' => $vendorDir . '/gettext/gettext/src/Utils/PhpFunctionsScanner.php',
    'Gettext\\Utils\\StringReader' => $vendorDir . '/gettext/gettext/src/Utils/StringReader.php',
    'JaimePerez\\TwigConfigurableI18n\\Twig\\Environment' => $vendorDir . '/jaimeperez/twig-configurable-i18n/src/Twig/Environment.php',
    'JaimePerez\\TwigConfigurableI18n\\Twig\\Extensions\\Extension\\I18n' => $vendorDir . '/jaimeperez/twig-configurable-i18n/src/Twig/Extensions/Extension/I18n.php',
    'JaimePerez\\TwigConfigurableI18n\\Twig\\Extensions\\Node\\Trans' => $vendorDir . '/jaimeperez/twig-configurable-i18n/src/Twig/Extensions/Node/Trans.php',
    'JaimePerez\\TwigConfigurableI18n\\Twig\\Extensions\\TokenParser\\Trans' => $vendorDir . '/jaimeperez/twig-configurable-i18n/src/Twig/Extensions/TokenParser/Trans.php',
    'Psr\\Log\\AbstractLogger' => $vendorDir . '/psr/log/Psr/Log/AbstractLogger.php',
    'Psr\\Log\\InvalidArgumentException' => $vendorDir . '/psr/log/Psr/Log/InvalidArgumentException.php',
    'Psr\\Log\\LogLevel' => $vendorDir . '/psr/log/Psr/Log/LogLevel.php',
    'Psr\\Log\\LoggerAwareInterface' => $vendorDir . '/psr/log/Psr/Log/LoggerAwareInterface.php',
    'Psr\\Log\\LoggerAwareTrait' => $vendorDir . '/psr/log/Psr/Log/LoggerAwareTrait.php',
    'Psr\\Log\\LoggerInterface' => $vendorDir . '/psr/log/Psr/Log/LoggerInterface.php',
    'Psr\\Log\\LoggerTrait' => $vendorDir . '/psr/log/Psr/Log/LoggerTrait.php',
    'Psr\\Log\\NullLogger' => $vendorDir . '/psr/log/Psr/Log/NullLogger.php',
    'Psr\\Log\\Test\\DummyTest' => $vendorDir . '/psr/log/Psr/Log/Test/LoggerInterfaceTest.php',
    'Psr\\Log\\Test\\LoggerInterfaceTest' => $vendorDir . '/psr/log/Psr/Log/Test/LoggerInterfaceTest.php',
    'RobRichards\\XMLSecLibs\\XMLSecEnc' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecEnc.php',
    'RobRichards\\XMLSecLibs\\XMLSecurityDSig' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityDSig.php',
    'RobRichards\\XMLSecLibs\\XMLSecurityKey' => $vendorDir . '/robrichards/xmlseclibs/src/XMLSecurityKey.php',
    'SAML2\\ArtifactResolve' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/ArtifactResolve.php',
    'SAML2\\ArtifactResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/ArtifactResponse.php',
    'SAML2\\Assertion' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion.php',
    'SAML2\\Assertion\\Decrypter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Decrypter.php',
    'SAML2\\Assertion\\Exception\\InvalidAssertionException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/InvalidAssertionException.php',
    'SAML2\\Assertion\\Exception\\InvalidSubjectConfirmationException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/InvalidSubjectConfirmationException.php',
    'SAML2\\Assertion\\Exception\\NotDecryptedException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Exception/NotDecryptedException.php',
    'SAML2\\Assertion\\Processor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Processor.php',
    'SAML2\\Assertion\\ProcessorBuilder' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/ProcessorBuilder.php',
    'SAML2\\Assertion\\Transformer\\DecodeBase64Transformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/DecodeBase64Transformer.php',
    'SAML2\\Assertion\\Transformer\\NameIdDecryptionTransformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/NameIdDecryptionTransformer.php',
    'SAML2\\Assertion\\Transformer\\Transformer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/Transformer.php',
    'SAML2\\Assertion\\Transformer\\TransformerChain' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Transformer/TransformerChain.php',
    'SAML2\\Assertion\\Validation\\AssertionConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/AssertionConstraintValidator.php',
    'SAML2\\Assertion\\Validation\\AssertionValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/AssertionValidator.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\NotBefore' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/NotBefore.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\NotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/NotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SessionNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SessionNotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SpIsValidAudience' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SpIsValidAudience.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationMethod' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationMethod.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotBefore' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotBefore.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationNotOnOrAfter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationNotOnOrAfter.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationRecipientMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationRecipientMatches.php',
    'SAML2\\Assertion\\Validation\\ConstraintValidator\\SubjectConfirmationResponseToMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/ConstraintValidator/SubjectConfirmationResponseToMatches.php',
    'SAML2\\Assertion\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/Result.php',
    'SAML2\\Assertion\\Validation\\SubjectConfirmationConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/SubjectConfirmationConstraintValidator.php',
    'SAML2\\Assertion\\Validation\\SubjectConfirmationValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Assertion/Validation/SubjectConfirmationValidator.php',
    'SAML2\\AttributeQuery' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/AttributeQuery.php',
    'SAML2\\AuthnRequest' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/AuthnRequest.php',
    'SAML2\\Binding' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Binding.php',
    'SAML2\\Certificate\\Exception\\InvalidCertificateStructureException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/InvalidCertificateStructureException.php',
    'SAML2\\Certificate\\Exception\\InvalidKeyUsageException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/InvalidKeyUsageException.php',
    'SAML2\\Certificate\\Exception\\NoKeysFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Exception/NoKeysFoundException.php',
    'SAML2\\Certificate\\Fingerprint' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Fingerprint.php',
    'SAML2\\Certificate\\FingerprintCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/FingerprintCollection.php',
    'SAML2\\Certificate\\FingerprintLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/FingerprintLoader.php',
    'SAML2\\Certificate\\Key' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/Key.php',
    'SAML2\\Certificate\\KeyCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/KeyCollection.php',
    'SAML2\\Certificate\\KeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/KeyLoader.php',
    'SAML2\\Certificate\\PrivateKey' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/PrivateKey.php',
    'SAML2\\Certificate\\PrivateKeyLoader' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/PrivateKeyLoader.php',
    'SAML2\\Certificate\\X509' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Certificate/X509.php',
    'SAML2\\Compat\\AbstractContainer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/AbstractContainer.php',
    'SAML2\\Compat\\ContainerSingleton' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/ContainerSingleton.php',
    'SAML2\\Compat\\MockContainer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/MockContainer.php',
    'SAML2\\Compat\\Ssp\\Container' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/Ssp/Container.php',
    'SAML2\\Compat\\Ssp\\Logger' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Compat/Ssp/Logger.php',
    'SAML2\\Configuration\\ArrayAdapter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ArrayAdapter.php',
    'SAML2\\Configuration\\CertificateProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/CertificateProvider.php',
    'SAML2\\Configuration\\DecryptionProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/DecryptionProvider.php',
    'SAML2\\Configuration\\Destination' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/Destination.php',
    'SAML2\\Configuration\\EntityIdProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/EntityIdProvider.php',
    'SAML2\\Configuration\\IdentityProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/IdentityProvider.php',
    'SAML2\\Configuration\\IdentityProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/IdentityProviderAware.php',
    'SAML2\\Configuration\\PrivateKey' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/PrivateKey.php',
    'SAML2\\Configuration\\Queryable' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/Queryable.php',
    'SAML2\\Configuration\\ServiceProvider' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ServiceProvider.php',
    'SAML2\\Configuration\\ServiceProviderAware' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/ServiceProviderAware.php',
    'SAML2\\Configuration\\SimpleSAMLConverter' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Configuration/SimpleSAMLConverter.php',
    'SAML2\\Constants' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Constants.php',
    'SAML2\\DOMDocumentFactory' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/DOMDocumentFactory.php',
    'SAML2\\EncryptedAssertion' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/EncryptedAssertion.php',
    'SAML2\\Exception\\InvalidArgumentException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/InvalidArgumentException.php',
    'SAML2\\Exception\\RuntimeException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/RuntimeException.php',
    'SAML2\\Exception\\Throwable' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/Throwable.php',
    'SAML2\\Exception\\UnparseableXmlException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Exception/UnparseableXmlException.php',
    'SAML2\\HTTPArtifact' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPArtifact.php',
    'SAML2\\HTTPPost' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPPost.php',
    'SAML2\\HTTPRedirect' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/HTTPRedirect.php',
    'SAML2\\LogoutRequest' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/LogoutRequest.php',
    'SAML2\\LogoutResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/LogoutResponse.php',
    'SAML2\\Message' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Message.php',
    'SAML2\\Request' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Request.php',
    'SAML2\\Response' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response.php',
    'SAML2\\Response\\Exception\\InvalidResponseException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/InvalidResponseException.php',
    'SAML2\\Response\\Exception\\InvalidSignatureException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/InvalidSignatureException.php',
    'SAML2\\Response\\Exception\\NoAssertionsFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/NoAssertionsFoundException.php',
    'SAML2\\Response\\Exception\\PreconditionNotMetException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/PreconditionNotMetException.php',
    'SAML2\\Response\\Exception\\UnencryptedAssertionFoundException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/UnencryptedAssertionFoundException.php',
    'SAML2\\Response\\Exception\\UnsignedResponseException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Exception/UnsignedResponseException.php',
    'SAML2\\Response\\Processor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Processor.php',
    'SAML2\\Response\\Validation\\ConstraintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator.php',
    'SAML2\\Response\\Validation\\ConstraintValidator\\DestinationMatches' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator/DestinationMatches.php',
    'SAML2\\Response\\Validation\\ConstraintValidator\\IsSuccessful' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/ConstraintValidator/IsSuccessful.php',
    'SAML2\\Response\\Validation\\PreconditionValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/PreconditionValidator.php',
    'SAML2\\Response\\Validation\\Result' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/Result.php',
    'SAML2\\Response\\Validation\\Validator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Response/Validation/Validator.php',
    'SAML2\\SOAP' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SOAP.php',
    'SAML2\\SOAPClient' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SOAPClient.php',
    'SAML2\\Signature\\AbstractChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/AbstractChainedValidator.php',
    'SAML2\\Signature\\ChainedValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ChainedValidator.php',
    'SAML2\\Signature\\FingerprintValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/FingerprintValidator.php',
    'SAML2\\Signature\\MissingConfigurationException' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/MissingConfigurationException.php',
    'SAML2\\Signature\\PublicKeyValidator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/PublicKeyValidator.php',
    'SAML2\\Signature\\Validator' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/Validator.php',
    'SAML2\\Signature\\ValidatorChain' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ValidatorChain.php',
    'SAML2\\Signature\\ValidatorInterface' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Signature/ValidatorInterface.php',
    'SAML2\\SignedElement' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SignedElement.php',
    'SAML2\\SignedElementHelper' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SignedElementHelper.php',
    'SAML2\\StatusResponse' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/StatusResponse.php',
    'SAML2\\SubjectQuery' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/SubjectQuery.php',
    'SAML2\\Utilities\\ArrayCollection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/ArrayCollection.php',
    'SAML2\\Utilities\\Certificate' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Certificate.php',
    'SAML2\\Utilities\\Collection' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Collection.php',
    'SAML2\\Utilities\\File' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/File.php',
    'SAML2\\Utilities\\Temporal' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utilities/Temporal.php',
    'SAML2\\Utils' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/Utils.php',
    'SAML2\\XML\\Chunk' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/Chunk.php',
    'SAML2\\XML\\alg\\Common' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/alg/Common.php',
    'SAML2\\XML\\alg\\DigestMethod' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/alg/DigestMethod.php',
    'SAML2\\XML\\alg\\SigningMethod' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/alg/SigningMethod.php',
    'SAML2\\XML\\ds\\KeyInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/KeyInfo.php',
    'SAML2\\XML\\ds\\KeyName' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/KeyName.php',
    'SAML2\\XML\\ds\\X509Certificate' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/X509Certificate.php',
    'SAML2\\XML\\ds\\X509Data' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ds/X509Data.php',
    'SAML2\\XML\\ecp\\Response' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/ecp/Response.php',
    'SAML2\\XML\\md\\AdditionalMetadataLocation' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AdditionalMetadataLocation.php',
    'SAML2\\XML\\md\\AffiliationDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AffiliationDescriptor.php',
    'SAML2\\XML\\md\\AttributeAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AttributeAuthorityDescriptor.php',
    'SAML2\\XML\\md\\AttributeConsumingService' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AttributeConsumingService.php',
    'SAML2\\XML\\md\\AuthnAuthorityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/AuthnAuthorityDescriptor.php',
    'SAML2\\XML\\md\\ContactPerson' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/ContactPerson.php',
    'SAML2\\XML\\md\\EndpointType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EndpointType.php',
    'SAML2\\XML\\md\\EntitiesDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EntitiesDescriptor.php',
    'SAML2\\XML\\md\\EntityDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/EntityDescriptor.php',
    'SAML2\\XML\\md\\Extensions' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/Extensions.php',
    'SAML2\\XML\\md\\IDPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/IDPSSODescriptor.php',
    'SAML2\\XML\\md\\IndexedEndpointType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/IndexedEndpointType.php',
    'SAML2\\XML\\md\\KeyDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/KeyDescriptor.php',
    'SAML2\\XML\\md\\Organization' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/Organization.php',
    'SAML2\\XML\\md\\PDPDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/PDPDescriptor.php',
    'SAML2\\XML\\md\\RequestedAttribute' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/RequestedAttribute.php',
    'SAML2\\XML\\md\\RoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/RoleDescriptor.php',
    'SAML2\\XML\\md\\SPSSODescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/SPSSODescriptor.php',
    'SAML2\\XML\\md\\SSODescriptorType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/SSODescriptorType.php',
    'SAML2\\XML\\md\\UnknownRoleDescriptor' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/md/UnknownRoleDescriptor.php',
    'SAML2\\XML\\mdattr\\EntityAttributes' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdattr/EntityAttributes.php',
    'SAML2\\XML\\mdrpi\\Common' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/Common.php',
    'SAML2\\XML\\mdrpi\\PublicationInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/PublicationInfo.php',
    'SAML2\\XML\\mdrpi\\RegistrationInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdrpi/RegistrationInfo.php',
    'SAML2\\XML\\mdui\\Common' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/Common.php',
    'SAML2\\XML\\mdui\\DiscoHints' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/DiscoHints.php',
    'SAML2\\XML\\mdui\\Keywords' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/Keywords.php',
    'SAML2\\XML\\mdui\\Logo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/Logo.php',
    'SAML2\\XML\\mdui\\UIInfo' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/mdui/UIInfo.php',
    'SAML2\\XML\\saml\\Attribute' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/Attribute.php',
    'SAML2\\XML\\saml\\AttributeValue' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/AttributeValue.php',
    'SAML2\\XML\\saml\\BaseIDType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/BaseIDType.php',
    'SAML2\\XML\\saml\\Issuer' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/Issuer.php',
    'SAML2\\XML\\saml\\NameID' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/NameID.php',
    'SAML2\\XML\\saml\\NameIDType' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/NameIDType.php',
    'SAML2\\XML\\saml\\SubjectConfirmation' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/SubjectConfirmation.php',
    'SAML2\\XML\\saml\\SubjectConfirmationData' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/saml/SubjectConfirmationData.php',
    'SAML2\\XML\\samlp\\Extensions' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/samlp/Extensions.php',
    'SAML2\\XML\\shibmd\\Scope' => $vendorDir . '/simplesamlphp/saml2/src/SAML2/XML/shibmd/Scope.php',
    'SimpleSAML\\Auth\\Simple' => $baseDir . '/lib/SimpleSAML/Auth/Simple.php',
    'SimpleSAML\\Auth\\TimeLimitedToken' => $baseDir . '/lib/SimpleSAML/Auth/TimeLimitedToken.php',
    'SimpleSAML\\Bindings\\Shib13\\Artifact' => $baseDir . '/lib/SimpleSAML/Bindings/Shib13/Artifact.php',
    'SimpleSAML\\Bindings\\Shib13\\HTTPPost' => $baseDir . '/lib/SimpleSAML/Bindings/Shib13/HTTPPost.php',
    'SimpleSAML\\Database' => $baseDir . '/lib/SimpleSAML/Database.php',
    'SimpleSAML\\Error\\CannotSetCookie' => $baseDir . '/lib/SimpleSAML/Error/CannotSetCookie.php',
    'SimpleSAML\\Error\\ConfigurationError' => $baseDir . '/lib/SimpleSAML/Error/ConfigurationError.php',
    'SimpleSAML\\Error\\CriticalConfigurationError' => $baseDir . '/lib/SimpleSAML/Error/CriticalConfigurationError.php',
    'SimpleSAML\\Error\\ErrorCodes' => $baseDir . '/lib/SimpleSAML/Error/ErrorCodes.php',
    'SimpleSAML\\IdP\\IFrameLogoutHandler' => $baseDir . '/lib/SimpleSAML/IdP/IFrameLogoutHandler.php',
    'SimpleSAML\\IdP\\LogoutHandlerInterface' => $baseDir . '/lib/SimpleSAML/IdP/LogoutHandlerInterface.php',
    'SimpleSAML\\IdP\\TraditionalLogoutHandler' => $baseDir . '/lib/SimpleSAML/IdP/TraditionalLogoutHandler.php',
    'SimpleSAML\\Locale\\Language' => $baseDir . '/lib/SimpleSAML/Locale/Language.php',
    'SimpleSAML\\Locale\\Localization' => $baseDir . '/lib/SimpleSAML/Locale/Localization.php',
    'SimpleSAML\\Locale\\Translate' => $baseDir . '/lib/SimpleSAML/Locale/Translate.php',
    'SimpleSAML\\Logger' => $baseDir . '/lib/SimpleSAML/Logger.php',
    'SimpleSAML\\Logger\\ErrorLogLoggingHandler' => $baseDir . '/lib/SimpleSAML/Logger/ErrorLogLoggingHandler.php',
    'SimpleSAML\\Logger\\FileLoggingHandler' => $baseDir . '/lib/SimpleSAML/Logger/FileLoggingHandler.php',
    'SimpleSAML\\Logger\\LoggingHandlerInterface' => $baseDir . '/lib/SimpleSAML/Logger/LoggingHandlerInterface.php',
    'SimpleSAML\\Logger\\StandardErrorLoggingHandler' => $baseDir . '/lib/SimpleSAML/Logger/StandardErrorLoggingHandler.php',
    'SimpleSAML\\Logger\\SyslogLoggingHandler' => $baseDir . '/lib/SimpleSAML/Logger/SyslogLoggingHandler.php',
    'SimpleSAML\\Metadata\\Sources\\MDQ' => $baseDir . '/lib/SimpleSAML/Metadata/Sources/MDQ.php',
    'SimpleSAML\\Module' => $baseDir . '/lib/SimpleSAML/Module.php',
    'SimpleSAML\\SessionHandler' => $baseDir . '/lib/SimpleSAML/SessionHandler.php',
    'SimpleSAML\\SessionHandlerCookie' => $baseDir . '/lib/SimpleSAML/SessionHandlerCookie.php',
    'SimpleSAML\\SessionHandlerPHP' => $baseDir . '/lib/SimpleSAML/SessionHandlerPHP.php',
    'SimpleSAML\\SessionHandlerStore' => $baseDir . '/lib/SimpleSAML/SessionHandlerStore.php',
    'SimpleSAML\\Store' => $baseDir . '/lib/SimpleSAML/Store.php',
    'SimpleSAML\\Store\\Memcache' => $baseDir . '/lib/SimpleSAML/Store/Memcache.php',
    'SimpleSAML\\Store\\Redis' => $baseDir . '/lib/SimpleSAML/Store/Redis.php',
    'SimpleSAML\\Store\\SQL' => $baseDir . '/lib/SimpleSAML/Store/SQL.php',
    'SimpleSAML\\Utils\\Arrays' => $baseDir . '/lib/SimpleSAML/Utils/Arrays.php',
    'SimpleSAML\\Utils\\Attributes' => $baseDir . '/lib/SimpleSAML/Utils/Attributes.php',
    'SimpleSAML\\Utils\\Auth' => $baseDir . '/lib/SimpleSAML/Utils/Auth.php',
    'SimpleSAML\\Utils\\ClearableState' => $baseDir . '/lib/SimpleSAML/Utils/ClearableState.php',
    'SimpleSAML\\Utils\\Config' => $baseDir . '/lib/SimpleSAML/Utils/Config.php',
    'SimpleSAML\\Utils\\Config\\Metadata' => $baseDir . '/lib/SimpleSAML/Utils/Config/Metadata.php',
    'SimpleSAML\\Utils\\Crypto' => $baseDir . '/lib/SimpleSAML/Utils/Crypto.php',
    'SimpleSAML\\Utils\\HTTP' => $baseDir . '/lib/SimpleSAML/Utils/HTTP.php',
    'SimpleSAML\\Utils\\Net' => $baseDir . '/lib/SimpleSAML/Utils/Net.php',
    'SimpleSAML\\Utils\\Random' => $baseDir . '/lib/SimpleSAML/Utils/Random.php',
    'SimpleSAML\\Utils\\System' => $baseDir . '/lib/SimpleSAML/Utils/System.php',
    'SimpleSAML\\Utils\\Time' => $baseDir . '/lib/SimpleSAML/Utils/Time.php',
    'SimpleSAML\\Utils\\XML' => $baseDir . '/lib/SimpleSAML/Utils/XML.php',
    'SimpleSAML\\XHTML\\TemplateControllerInterface' => $baseDir . '/lib/SimpleSAML/XHTML/TemplateControllerInterface.php',
    'SimpleSAML\\XML\\Errors' => $baseDir . '/lib/SimpleSAML/XML/Errors.php',
    'SimpleSAML\\XML\\Parser' => $baseDir . '/lib/SimpleSAML/XML/Parser.php',
    'SimpleSAML\\XML\\Shib13\\AuthnRequest' => $baseDir . '/lib/SimpleSAML/XML/Shib13/AuthnRequest.php',
    'SimpleSAML\\XML\\Shib13\\AuthnResponse' => $baseDir . '/lib/SimpleSAML/XML/Shib13/AuthnResponse.php',
    'SimpleSAML\\XML\\Signer' => $baseDir . '/lib/SimpleSAML/XML/Signer.php',
    'SimpleSAML\\XML\\Validator' => $baseDir . '/lib/SimpleSAML/XML/Validator.php',
    'Twig\\Cache\\CacheInterface' => $vendorDir . '/twig/twig/src/Cache/CacheInterface.php',
    'Twig\\Cache\\FilesystemCache' => $vendorDir . '/twig/twig/src/Cache/FilesystemCache.php',
    'Twig\\Cache\\NullCache' => $vendorDir . '/twig/twig/src/Cache/NullCache.php',
    'Twig\\Compiler' => $vendorDir . '/twig/twig/src/Compiler.php',
    'Twig\\Environment' => $vendorDir . '/twig/twig/src/Environment.php',
    'Twig\\Error\\Error' => $vendorDir . '/twig/twig/src/Error/Error.php',
    'Twig\\Error\\LoaderError' => $vendorDir . '/twig/twig/src/Error/LoaderError.php',
    'Twig\\Error\\RuntimeError' => $vendorDir . '/twig/twig/src/Error/RuntimeError.php',
    'Twig\\Error\\SyntaxError' => $vendorDir . '/twig/twig/src/Error/SyntaxError.php',
    'Twig\\ExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser.php',
    'Twig\\Extension\\AbstractExtension' => $vendorDir . '/twig/twig/src/Extension/AbstractExtension.php',
    'Twig\\Extension\\CoreExtension' => $vendorDir . '/twig/twig/src/Extension/CoreExtension.php',
    'Twig\\Extension\\DebugExtension' => $vendorDir . '/twig/twig/src/Extension/DebugExtension.php',
    'Twig\\Extension\\EscaperExtension' => $vendorDir . '/twig/twig/src/Extension/EscaperExtension.php',
    'Twig\\Extension\\ExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/ExtensionInterface.php',
    'Twig\\Extension\\GlobalsInterface' => $vendorDir . '/twig/twig/src/Extension/GlobalsInterface.php',
    'Twig\\Extension\\InitRuntimeInterface' => $vendorDir . '/twig/twig/src/Extension/InitRuntimeInterface.php',
    'Twig\\Extension\\OptimizerExtension' => $vendorDir . '/twig/twig/src/Extension/OptimizerExtension.php',
    'Twig\\Extension\\ProfilerExtension' => $vendorDir . '/twig/twig/src/Extension/ProfilerExtension.php',
    'Twig\\Extension\\RuntimeExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/RuntimeExtensionInterface.php',
    'Twig\\Extension\\SandboxExtension' => $vendorDir . '/twig/twig/src/Extension/SandboxExtension.php',
    'Twig\\Extension\\StagingExtension' => $vendorDir . '/twig/twig/src/Extension/StagingExtension.php',
    'Twig\\Extension\\StringLoaderExtension' => $vendorDir . '/twig/twig/src/Extension/StringLoaderExtension.php',
    'Twig\\Extensions\\ArrayExtension' => $vendorDir . '/twig/extensions/src/ArrayExtension.php',
    'Twig\\Extensions\\DateExtension' => $vendorDir . '/twig/extensions/src/DateExtension.php',
    'Twig\\Extensions\\I18nExtension' => $vendorDir . '/twig/extensions/src/I18nExtension.php',
    'Twig\\Extensions\\IntlExtension' => $vendorDir . '/twig/extensions/src/IntlExtension.php',
    'Twig\\Extensions\\Node\\TransNode' => $vendorDir . '/twig/extensions/src/Node/TransNode.php',
    'Twig\\Extensions\\TextExtension' => $vendorDir . '/twig/extensions/src/TextExtension.php',
    'Twig\\Extensions\\TokenParser\\TransTokenParser' => $vendorDir . '/twig/extensions/src/TokenParser/TransTokenParser.php',
    'Twig\\FileExtensionEscapingStrategy' => $vendorDir . '/twig/twig/src/FileExtensionEscapingStrategy.php',
    'Twig\\Lexer' => $vendorDir . '/twig/twig/src/Lexer.php',
    'Twig\\Loader\\ArrayLoader' => $vendorDir . '/twig/twig/src/Loader/ArrayLoader.php',
    'Twig\\Loader\\ChainLoader' => $vendorDir . '/twig/twig/src/Loader/ChainLoader.php',
    'Twig\\Loader\\ExistsLoaderInterface' => $vendorDir . '/twig/twig/src/Loader/ExistsLoaderInterface.php',
    'Twig\\Loader\\FilesystemLoader' => $vendorDir . '/twig/twig/src/Loader/FilesystemLoader.php',
    'Twig\\Loader\\LoaderInterface' => $vendorDir . '/twig/twig/src/Loader/LoaderInterface.php',
    'Twig\\Loader\\SourceContextLoaderInterface' => $vendorDir . '/twig/twig/src/Loader/SourceContextLoaderInterface.php',
    'Twig\\Markup' => $vendorDir . '/twig/twig/src/Markup.php',
    'Twig\\NodeTraverser' => $vendorDir . '/twig/twig/src/NodeTraverser.php',
    'Twig\\NodeVisitor\\AbstractNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/AbstractNodeVisitor.php',
    'Twig\\NodeVisitor\\EscaperNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/EscaperNodeVisitor.php',
    'Twig\\NodeVisitor\\NodeVisitorInterface' => $vendorDir . '/twig/twig/src/NodeVisitor/NodeVisitorInterface.php',
    'Twig\\NodeVisitor\\OptimizerNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/OptimizerNodeVisitor.php',
    'Twig\\NodeVisitor\\SafeAnalysisNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SafeAnalysisNodeVisitor.php',
    'Twig\\NodeVisitor\\SandboxNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SandboxNodeVisitor.php',
    'Twig\\Node\\AutoEscapeNode' => $vendorDir . '/twig/twig/src/Node/AutoEscapeNode.php',
    'Twig\\Node\\BlockNode' => $vendorDir . '/twig/twig/src/Node/BlockNode.php',
    'Twig\\Node\\BlockReferenceNode' => $vendorDir . '/twig/twig/src/Node/BlockReferenceNode.php',
    'Twig\\Node\\BodyNode' => $vendorDir . '/twig/twig/src/Node/BodyNode.php',
    'Twig\\Node\\CheckSecurityNode' => $vendorDir . '/twig/twig/src/Node/CheckSecurityNode.php',
    'Twig\\Node\\DoNode' => $vendorDir . '/twig/twig/src/Node/DoNode.php',
    'Twig\\Node\\EmbedNode' => $vendorDir . '/twig/twig/src/Node/EmbedNode.php',
    'Twig\\Node\\Expression\\AbstractExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AbstractExpression.php',
    'Twig\\Node\\Expression\\ArrayExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ArrayExpression.php',
    'Twig\\Node\\Expression\\AssignNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AssignNameExpression.php',
    'Twig\\Node\\Expression\\Binary\\AbstractBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AbstractBinary.php',
    'Twig\\Node\\Expression\\Binary\\AddBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AddBinary.php',
    'Twig\\Node\\Expression\\Binary\\AndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseAndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseAndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseOrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseOrBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseXorBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseXorBinary.php',
    'Twig\\Node\\Expression\\Binary\\ConcatBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ConcatBinary.php',
    'Twig\\Node\\Expression\\Binary\\DivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/DivBinary.php',
    'Twig\\Node\\Expression\\Binary\\EndsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EndsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\EqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\FloorDivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/FloorDivBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\InBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/InBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\MatchesBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MatchesBinary.php',
    'Twig\\Node\\Expression\\Binary\\ModBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ModBinary.php',
    'Twig\\Node\\Expression\\Binary\\MulBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MulBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotInBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotInBinary.php',
    'Twig\\Node\\Expression\\Binary\\OrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/OrBinary.php',
    'Twig\\Node\\Expression\\Binary\\PowerBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/PowerBinary.php',
    'Twig\\Node\\Expression\\Binary\\RangeBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/RangeBinary.php',
    'Twig\\Node\\Expression\\Binary\\StartsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/StartsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\SubBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SubBinary.php',
    'Twig\\Node\\Expression\\BlockReferenceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/BlockReferenceExpression.php',
    'Twig\\Node\\Expression\\CallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/CallExpression.php',
    'Twig\\Node\\Expression\\ConditionalExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConditionalExpression.php',
    'Twig\\Node\\Expression\\ConstantExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConstantExpression.php',
    'Twig\\Node\\Expression\\FilterExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FilterExpression.php',
    'Twig\\Node\\Expression\\Filter\\DefaultFilter' => $vendorDir . '/twig/twig/src/Node/Expression/Filter/DefaultFilter.php',
    'Twig\\Node\\Expression\\FunctionExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionExpression.php',
    'Twig\\Node\\Expression\\GetAttrExpression' => $vendorDir . '/twig/twig/src/Node/Expression/GetAttrExpression.php',
    'Twig\\Node\\Expression\\MethodCallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/MethodCallExpression.php',
    'Twig\\Node\\Expression\\NameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NameExpression.php',
    'Twig\\Node\\Expression\\NullCoalesceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NullCoalesceExpression.php',
    'Twig\\Node\\Expression\\ParentExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ParentExpression.php',
    'Twig\\Node\\Expression\\TempNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TempNameExpression.php',
    'Twig\\Node\\Expression\\TestExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TestExpression.php',
    'Twig\\Node\\Expression\\Test\\ConstantTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/ConstantTest.php',
    'Twig\\Node\\Expression\\Test\\DefinedTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DefinedTest.php',
    'Twig\\Node\\Expression\\Test\\DivisiblebyTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DivisiblebyTest.php',
    'Twig\\Node\\Expression\\Test\\EvenTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/EvenTest.php',
    'Twig\\Node\\Expression\\Test\\NullTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/NullTest.php',
    'Twig\\Node\\Expression\\Test\\OddTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/OddTest.php',
    'Twig\\Node\\Expression\\Test\\SameasTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/SameasTest.php',
    'Twig\\Node\\Expression\\Unary\\AbstractUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/AbstractUnary.php',
    'Twig\\Node\\Expression\\Unary\\NegUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NegUnary.php',
    'Twig\\Node\\Expression\\Unary\\NotUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NotUnary.php',
    'Twig\\Node\\Expression\\Unary\\PosUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/PosUnary.php',
    'Twig\\Node\\FlushNode' => $vendorDir . '/twig/twig/src/Node/FlushNode.php',
    'Twig\\Node\\ForLoopNode' => $vendorDir . '/twig/twig/src/Node/ForLoopNode.php',
    'Twig\\Node\\ForNode' => $vendorDir . '/twig/twig/src/Node/ForNode.php',
    'Twig\\Node\\IfNode' => $vendorDir . '/twig/twig/src/Node/IfNode.php',
    'Twig\\Node\\ImportNode' => $vendorDir . '/twig/twig/src/Node/ImportNode.php',
    'Twig\\Node\\IncludeNode' => $vendorDir . '/twig/twig/src/Node/IncludeNode.php',
    'Twig\\Node\\MacroNode' => $vendorDir . '/twig/twig/src/Node/MacroNode.php',
    'Twig\\Node\\ModuleNode' => $vendorDir . '/twig/twig/src/Node/ModuleNode.php',
    'Twig\\Node\\Node' => $vendorDir . '/twig/twig/src/Node/Node.php',
    'Twig\\Node\\NodeCaptureInterface' => $vendorDir . '/twig/twig/src/Node/NodeCaptureInterface.php',
    'Twig\\Node\\NodeOutputInterface' => $vendorDir . '/twig/twig/src/Node/NodeOutputInterface.php',
    'Twig\\Node\\PrintNode' => $vendorDir . '/twig/twig/src/Node/PrintNode.php',
    'Twig\\Node\\SandboxNode' => $vendorDir . '/twig/twig/src/Node/SandboxNode.php',
    'Twig\\Node\\SandboxedPrintNode' => $vendorDir . '/twig/twig/src/Node/SandboxedPrintNode.php',
    'Twig\\Node\\SetNode' => $vendorDir . '/twig/twig/src/Node/SetNode.php',
    'Twig\\Node\\SetTempNode' => $vendorDir . '/twig/twig/src/Node/SetTempNode.php',
    'Twig\\Node\\SpacelessNode' => $vendorDir . '/twig/twig/src/Node/SpacelessNode.php',
    'Twig\\Node\\TextNode' => $vendorDir . '/twig/twig/src/Node/TextNode.php',
    'Twig\\Node\\WithNode' => $vendorDir . '/twig/twig/src/Node/WithNode.php',
    'Twig\\Parser' => $vendorDir . '/twig/twig/src/Parser.php',
    'Twig\\Profiler\\Dumper\\BaseDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BaseDumper.php',
    'Twig\\Profiler\\Dumper\\BlackfireDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BlackfireDumper.php',
    'Twig\\Profiler\\Dumper\\HtmlDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/HtmlDumper.php',
    'Twig\\Profiler\\Dumper\\TextDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/TextDumper.php',
    'Twig\\Profiler\\NodeVisitor\\ProfilerNodeVisitor' => $vendorDir . '/twig/twig/src/Profiler/NodeVisitor/ProfilerNodeVisitor.php',
    'Twig\\Profiler\\Node\\EnterProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/EnterProfileNode.php',
    'Twig\\Profiler\\Node\\LeaveProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/LeaveProfileNode.php',
    'Twig\\Profiler\\Profile' => $vendorDir . '/twig/twig/src/Profiler/Profile.php',
    'Twig\\RuntimeLoader\\ContainerRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/ContainerRuntimeLoader.php',
    'Twig\\RuntimeLoader\\FactoryRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/FactoryRuntimeLoader.php',
    'Twig\\RuntimeLoader\\RuntimeLoaderInterface' => $vendorDir . '/twig/twig/src/RuntimeLoader/RuntimeLoaderInterface.php',
    'Twig\\Sandbox\\SecurityError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFilterError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFilterError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFunctionError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFunctionError.php',
    'Twig\\Sandbox\\SecurityNotAllowedMethodError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedMethodError.php',
    'Twig\\Sandbox\\SecurityNotAllowedPropertyError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedPropertyError.php',
    'Twig\\Sandbox\\SecurityNotAllowedTagError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedTagError.php',
    'Twig\\Sandbox\\SecurityPolicy' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicy.php',
    'Twig\\Sandbox\\SecurityPolicyInterface' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicyInterface.php',
    'Twig\\Source' => $vendorDir . '/twig/twig/src/Source.php',
    'Twig\\Template' => $vendorDir . '/twig/twig/src/Template.php',
    'Twig\\TemplateWrapper' => $vendorDir . '/twig/twig/src/TemplateWrapper.php',
    'Twig\\Test\\IntegrationTestCase' => $vendorDir . '/twig/twig/src/Test/IntegrationTestCase.php',
    'Twig\\Test\\NodeTestCase' => $vendorDir . '/twig/twig/src/Test/NodeTestCase.php',
    'Twig\\Token' => $vendorDir . '/twig/twig/src/Token.php',
    'Twig\\TokenParser\\AbstractTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AbstractTokenParser.php',
    'Twig\\TokenParser\\AutoEscapeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AutoEscapeTokenParser.php',
    'Twig\\TokenParser\\BlockTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/BlockTokenParser.php',
    'Twig\\TokenParser\\DoTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/DoTokenParser.php',
    'Twig\\TokenParser\\EmbedTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/EmbedTokenParser.php',
    'Twig\\TokenParser\\ExtendsTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ExtendsTokenParser.php',
    'Twig\\TokenParser\\FilterTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FilterTokenParser.php',
    'Twig\\TokenParser\\FlushTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FlushTokenParser.php',
    'Twig\\TokenParser\\ForTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ForTokenParser.php',
    'Twig\\TokenParser\\FromTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FromTokenParser.php',
    'Twig\\TokenParser\\IfTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IfTokenParser.php',
    'Twig\\TokenParser\\ImportTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ImportTokenParser.php',
    'Twig\\TokenParser\\IncludeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IncludeTokenParser.php',
    'Twig\\TokenParser\\MacroTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/MacroTokenParser.php',
    'Twig\\TokenParser\\SandboxTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SandboxTokenParser.php',
    'Twig\\TokenParser\\SetTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SetTokenParser.php',
    'Twig\\TokenParser\\SpacelessTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SpacelessTokenParser.php',
    'Twig\\TokenParser\\TokenParserInterface' => $vendorDir . '/twig/twig/src/TokenParser/TokenParserInterface.php',
    'Twig\\TokenParser\\UseTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/UseTokenParser.php',
    'Twig\\TokenParser\\WithTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/WithTokenParser.php',
    'Twig\\TokenStream' => $vendorDir . '/twig/twig/src/TokenStream.php',
    'Twig\\TwigFilter' => $vendorDir . '/twig/twig/src/TwigFilter.php',
    'Twig\\TwigFunction' => $vendorDir . '/twig/twig/src/TwigFunction.php',
    'Twig\\TwigTest' => $vendorDir . '/twig/twig/src/TwigTest.php',
    'Twig\\Util\\DeprecationCollector' => $vendorDir . '/twig/twig/src/Util/DeprecationCollector.php',
    'Twig\\Util\\TemplateDirIterator' => $vendorDir . '/twig/twig/src/Util/TemplateDirIterator.php',
    'Twig_Autoloader' => $vendorDir . '/twig/twig/lib/Twig/Autoloader.php',
    'Twig_BaseNodeVisitor' => $vendorDir . '/twig/twig/lib/Twig/BaseNodeVisitor.php',
    'Twig_CacheInterface' => $vendorDir . '/twig/twig/lib/Twig/CacheInterface.php',
    'Twig_Cache_Filesystem' => $vendorDir . '/twig/twig/lib/Twig/Cache/Filesystem.php',
    'Twig_Cache_Null' => $vendorDir . '/twig/twig/lib/Twig/Cache/Null.php',
    'Twig_Compiler' => $vendorDir . '/twig/twig/lib/Twig/Compiler.php',
    'Twig_CompilerInterface' => $vendorDir . '/twig/twig/lib/Twig/CompilerInterface.php',
    'Twig_ContainerRuntimeLoader' => $vendorDir . '/twig/twig/lib/Twig/ContainerRuntimeLoader.php',
    'Twig_Environment' => $vendorDir . '/twig/twig/lib/Twig/Environment.php',
    'Twig_Error' => $vendorDir . '/twig/twig/lib/Twig/Error.php',
    'Twig_Error_Loader' => $vendorDir . '/twig/twig/lib/Twig/Error/Loader.php',
    'Twig_Error_Runtime' => $vendorDir . '/twig/twig/lib/Twig/Error/Runtime.php',
    'Twig_Error_Syntax' => $vendorDir . '/twig/twig/lib/Twig/Error/Syntax.php',
    'Twig_ExistsLoaderInterface' => $vendorDir . '/twig/twig/lib/Twig/ExistsLoaderInterface.php',
    'Twig_ExpressionParser' => $vendorDir . '/twig/twig/lib/Twig/ExpressionParser.php',
    'Twig_Extension' => $vendorDir . '/twig/twig/lib/Twig/Extension.php',
    'Twig_ExtensionInterface' => $vendorDir . '/twig/twig/lib/Twig/ExtensionInterface.php',
    'Twig_Extension_Core' => $vendorDir . '/twig/twig/lib/Twig/Extension/Core.php',
    'Twig_Extension_Debug' => $vendorDir . '/twig/twig/lib/Twig/Extension/Debug.php',
    'Twig_Extension_Escaper' => $vendorDir . '/twig/twig/lib/Twig/Extension/Escaper.php',
    'Twig_Extension_GlobalsInterface' => $vendorDir . '/twig/twig/lib/Twig/Extension/GlobalsInterface.php',
    'Twig_Extension_InitRuntimeInterface' => $vendorDir . '/twig/twig/lib/Twig/Extension/InitRuntimeInterface.php',
    'Twig_Extension_Optimizer' => $vendorDir . '/twig/twig/lib/Twig/Extension/Optimizer.php',
    'Twig_Extension_Profiler' => $vendorDir . '/twig/twig/lib/Twig/Extension/Profiler.php',
    'Twig_Extension_Sandbox' => $vendorDir . '/twig/twig/lib/Twig/Extension/Sandbox.php',
    'Twig_Extension_Staging' => $vendorDir . '/twig/twig/lib/Twig/Extension/Staging.php',
    'Twig_Extension_StringLoader' => $vendorDir . '/twig/twig/lib/Twig/Extension/StringLoader.php',
    'Twig_Extensions_Autoloader' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Autoloader.php',
    'Twig_Extensions_Extension_Array' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Extension/Array.php',
    'Twig_Extensions_Extension_Date' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Extension/Date.php',
    'Twig_Extensions_Extension_I18n' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Extension/I18n.php',
    'Twig_Extensions_Extension_Intl' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Extension/Intl.php',
    'Twig_Extensions_Extension_Text' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Extension/Text.php',
    'Twig_Extensions_Grammar' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar.php',
    'Twig_Extensions_GrammarInterface' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/GrammarInterface.php',
    'Twig_Extensions_Grammar_Arguments' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Arguments.php',
    'Twig_Extensions_Grammar_Array' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Array.php',
    'Twig_Extensions_Grammar_Body' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Body.php',
    'Twig_Extensions_Grammar_Boolean' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Boolean.php',
    'Twig_Extensions_Grammar_Constant' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Constant.php',
    'Twig_Extensions_Grammar_Expression' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Expression.php',
    'Twig_Extensions_Grammar_Hash' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Hash.php',
    'Twig_Extensions_Grammar_Number' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Number.php',
    'Twig_Extensions_Grammar_Optional' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Optional.php',
    'Twig_Extensions_Grammar_Switch' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Switch.php',
    'Twig_Extensions_Grammar_Tag' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Grammar/Tag.php',
    'Twig_Extensions_Node_Trans' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/Node/Trans.php',
    'Twig_Extensions_SimpleTokenParser' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/SimpleTokenParser.php',
    'Twig_Extensions_TokenParser_Trans' => $vendorDir . '/twig/extensions/lib/Twig/Extensions/TokenParser/Trans.php',
    'Twig_FactoryRuntimeLoader' => $vendorDir . '/twig/twig/lib/Twig/FactoryRuntimeLoader.php',
    'Twig_FileExtensionEscapingStrategy' => $vendorDir . '/twig/twig/lib/Twig/FileExtensionEscapingStrategy.php',
    'Twig_Filter' => $vendorDir . '/twig/twig/lib/Twig/Filter.php',
    'Twig_FilterCallableInterface' => $vendorDir . '/twig/twig/lib/Twig/FilterCallableInterface.php',
    'Twig_FilterInterface' => $vendorDir . '/twig/twig/lib/Twig/FilterInterface.php',
    'Twig_Filter_Function' => $vendorDir . '/twig/twig/lib/Twig/Filter/Function.php',
    'Twig_Filter_Method' => $vendorDir . '/twig/twig/lib/Twig/Filter/Method.php',
    'Twig_Filter_Node' => $vendorDir . '/twig/twig/lib/Twig/Filter/Node.php',
    'Twig_Function' => $vendorDir . '/twig/twig/lib/Twig/Function.php',
    'Twig_FunctionCallableInterface' => $vendorDir . '/twig/twig/lib/Twig/FunctionCallableInterface.php',
    'Twig_FunctionInterface' => $vendorDir . '/twig/twig/lib/Twig/FunctionInterface.php',
    'Twig_Function_Function' => $vendorDir . '/twig/twig/lib/Twig/Function/Function.php',
    'Twig_Function_Method' => $vendorDir . '/twig/twig/lib/Twig/Function/Method.php',
    'Twig_Function_Node' => $vendorDir . '/twig/twig/lib/Twig/Function/Node.php',
    'Twig_Lexer' => $vendorDir . '/twig/twig/lib/Twig/Lexer.php',
    'Twig_LexerInterface' => $vendorDir . '/twig/twig/lib/Twig/LexerInterface.php',
    'Twig_LoaderInterface' => $vendorDir . '/twig/twig/lib/Twig/LoaderInterface.php',
    'Twig_Loader_Array' => $vendorDir . '/twig/twig/lib/Twig/Loader/Array.php',
    'Twig_Loader_Chain' => $vendorDir . '/twig/twig/lib/Twig/Loader/Chain.php',
    'Twig_Loader_Filesystem' => $vendorDir . '/twig/twig/lib/Twig/Loader/Filesystem.php',
    'Twig_Loader_String' => $vendorDir . '/twig/twig/lib/Twig/Loader/String.php',
    'Twig_Markup' => $vendorDir . '/twig/twig/lib/Twig/Markup.php',
    'Twig_Node' => $vendorDir . '/twig/twig/lib/Twig/Node.php',
    'Twig_NodeCaptureInterface' => $vendorDir . '/twig/twig/lib/Twig/NodeCaptureInterface.php',
    'Twig_NodeInterface' => $vendorDir . '/twig/twig/lib/Twig/NodeInterface.php',
    'Twig_NodeOutputInterface' => $vendorDir . '/twig/twig/lib/Twig/NodeOutputInterface.php',
    'Twig_NodeTraverser' => $vendorDir . '/twig/twig/lib/Twig/NodeTraverser.php',
    'Twig_NodeVisitorInterface' => $vendorDir . '/twig/twig/lib/Twig/NodeVisitorInterface.php',
    'Twig_NodeVisitor_Escaper' => $vendorDir . '/twig/twig/lib/Twig/NodeVisitor/Escaper.php',
    'Twig_NodeVisitor_Optimizer' => $vendorDir . '/twig/twig/lib/Twig/NodeVisitor/Optimizer.php',
    'Twig_NodeVisitor_SafeAnalysis' => $vendorDir . '/twig/twig/lib/Twig/NodeVisitor/SafeAnalysis.php',
    'Twig_NodeVisitor_Sandbox' => $vendorDir . '/twig/twig/lib/Twig/NodeVisitor/Sandbox.php',
    'Twig_Node_AutoEscape' => $vendorDir . '/twig/twig/lib/Twig/Node/AutoEscape.php',
    'Twig_Node_Block' => $vendorDir . '/twig/twig/lib/Twig/Node/Block.php',
    'Twig_Node_BlockReference' => $vendorDir . '/twig/twig/lib/Twig/Node/BlockReference.php',
    'Twig_Node_Body' => $vendorDir . '/twig/twig/lib/Twig/Node/Body.php',
    'Twig_Node_CheckSecurity' => $vendorDir . '/twig/twig/lib/Twig/Node/CheckSecurity.php',
    'Twig_Node_Do' => $vendorDir . '/twig/twig/lib/Twig/Node/Do.php',
    'Twig_Node_Embed' => $vendorDir . '/twig/twig/lib/Twig/Node/Embed.php',
    'Twig_Node_Expression' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression.php',
    'Twig_Node_Expression_Array' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Array.php',
    'Twig_Node_Expression_AssignName' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/AssignName.php',
    'Twig_Node_Expression_Binary' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary.php',
    'Twig_Node_Expression_Binary_Add' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Add.php',
    'Twig_Node_Expression_Binary_And' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/And.php',
    'Twig_Node_Expression_Binary_BitwiseAnd' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/BitwiseAnd.php',
    'Twig_Node_Expression_Binary_BitwiseOr' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/BitwiseOr.php',
    'Twig_Node_Expression_Binary_BitwiseXor' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/BitwiseXor.php',
    'Twig_Node_Expression_Binary_Concat' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Concat.php',
    'Twig_Node_Expression_Binary_Div' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Div.php',
    'Twig_Node_Expression_Binary_EndsWith' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/EndsWith.php',
    'Twig_Node_Expression_Binary_Equal' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Equal.php',
    'Twig_Node_Expression_Binary_FloorDiv' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/FloorDiv.php',
    'Twig_Node_Expression_Binary_Greater' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Greater.php',
    'Twig_Node_Expression_Binary_GreaterEqual' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/GreaterEqual.php',
    'Twig_Node_Expression_Binary_In' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/In.php',
    'Twig_Node_Expression_Binary_Less' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Less.php',
    'Twig_Node_Expression_Binary_LessEqual' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/LessEqual.php',
    'Twig_Node_Expression_Binary_Matches' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Matches.php',
    'Twig_Node_Expression_Binary_Mod' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Mod.php',
    'Twig_Node_Expression_Binary_Mul' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Mul.php',
    'Twig_Node_Expression_Binary_NotEqual' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/NotEqual.php',
    'Twig_Node_Expression_Binary_NotIn' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/NotIn.php',
    'Twig_Node_Expression_Binary_Or' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Or.php',
    'Twig_Node_Expression_Binary_Power' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Power.php',
    'Twig_Node_Expression_Binary_Range' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Range.php',
    'Twig_Node_Expression_Binary_StartsWith' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/StartsWith.php',
    'Twig_Node_Expression_Binary_Sub' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Binary/Sub.php',
    'Twig_Node_Expression_BlockReference' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/BlockReference.php',
    'Twig_Node_Expression_Call' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Call.php',
    'Twig_Node_Expression_Conditional' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Conditional.php',
    'Twig_Node_Expression_Constant' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Constant.php',
    'Twig_Node_Expression_ExtensionReference' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/ExtensionReference.php',
    'Twig_Node_Expression_Filter' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Filter.php',
    'Twig_Node_Expression_Filter_Default' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Filter/Default.php',
    'Twig_Node_Expression_Function' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Function.php',
    'Twig_Node_Expression_GetAttr' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/GetAttr.php',
    'Twig_Node_Expression_MethodCall' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/MethodCall.php',
    'Twig_Node_Expression_Name' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Name.php',
    'Twig_Node_Expression_NullCoalesce' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/NullCoalesce.php',
    'Twig_Node_Expression_Parent' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Parent.php',
    'Twig_Node_Expression_TempName' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/TempName.php',
    'Twig_Node_Expression_Test' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test.php',
    'Twig_Node_Expression_Test_Constant' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Constant.php',
    'Twig_Node_Expression_Test_Defined' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Defined.php',
    'Twig_Node_Expression_Test_Divisibleby' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Divisibleby.php',
    'Twig_Node_Expression_Test_Even' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Even.php',
    'Twig_Node_Expression_Test_Null' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Null.php',
    'Twig_Node_Expression_Test_Odd' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Odd.php',
    'Twig_Node_Expression_Test_Sameas' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Test/Sameas.php',
    'Twig_Node_Expression_Unary' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Unary.php',
    'Twig_Node_Expression_Unary_Neg' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Unary/Neg.php',
    'Twig_Node_Expression_Unary_Not' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Unary/Not.php',
    'Twig_Node_Expression_Unary_Pos' => $vendorDir . '/twig/twig/lib/Twig/Node/Expression/Unary/Pos.php',
    'Twig_Node_Flush' => $vendorDir . '/twig/twig/lib/Twig/Node/Flush.php',
    'Twig_Node_For' => $vendorDir . '/twig/twig/lib/Twig/Node/For.php',
    'Twig_Node_ForLoop' => $vendorDir . '/twig/twig/lib/Twig/Node/ForLoop.php',
    'Twig_Node_If' => $vendorDir . '/twig/twig/lib/Twig/Node/If.php',
    'Twig_Node_Import' => $vendorDir . '/twig/twig/lib/Twig/Node/Import.php',
    'Twig_Node_Include' => $vendorDir . '/twig/twig/lib/Twig/Node/Include.php',
    'Twig_Node_Macro' => $vendorDir . '/twig/twig/lib/Twig/Node/Macro.php',
    'Twig_Node_Module' => $vendorDir . '/twig/twig/lib/Twig/Node/Module.php',
    'Twig_Node_Print' => $vendorDir . '/twig/twig/lib/Twig/Node/Print.php',
    'Twig_Node_Sandbox' => $vendorDir . '/twig/twig/lib/Twig/Node/Sandbox.php',
    'Twig_Node_SandboxedPrint' => $vendorDir . '/twig/twig/lib/Twig/Node/SandboxedPrint.php',
    'Twig_Node_Set' => $vendorDir . '/twig/twig/lib/Twig/Node/Set.php',
    'Twig_Node_SetTemp' => $vendorDir . '/twig/twig/lib/Twig/Node/SetTemp.php',
    'Twig_Node_Spaceless' => $vendorDir . '/twig/twig/lib/Twig/Node/Spaceless.php',
    'Twig_Node_Text' => $vendorDir . '/twig/twig/lib/Twig/Node/Text.php',
    'Twig_Node_With' => $vendorDir . '/twig/twig/lib/Twig/Node/With.php',
    'Twig_Parser' => $vendorDir . '/twig/twig/lib/Twig/Parser.php',
    'Twig_ParserInterface' => $vendorDir . '/twig/twig/lib/Twig/ParserInterface.php',
    'Twig_Profiler_Dumper_Base' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Dumper/Base.php',
    'Twig_Profiler_Dumper_Blackfire' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Dumper/Blackfire.php',
    'Twig_Profiler_Dumper_Html' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Dumper/Html.php',
    'Twig_Profiler_Dumper_Text' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Dumper/Text.php',
    'Twig_Profiler_NodeVisitor_Profiler' => $vendorDir . '/twig/twig/lib/Twig/Profiler/NodeVisitor/Profiler.php',
    'Twig_Profiler_Node_EnterProfile' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Node/EnterProfile.php',
    'Twig_Profiler_Node_LeaveProfile' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Node/LeaveProfile.php',
    'Twig_Profiler_Profile' => $vendorDir . '/twig/twig/lib/Twig/Profiler/Profile.php',
    'Twig_RuntimeLoaderInterface' => $vendorDir . '/twig/twig/lib/Twig/RuntimeLoaderInterface.php',
    'Twig_Sandbox_SecurityError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityError.php',
    'Twig_Sandbox_SecurityNotAllowedFilterError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityNotAllowedFilterError.php',
    'Twig_Sandbox_SecurityNotAllowedFunctionError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityNotAllowedFunctionError.php',
    'Twig_Sandbox_SecurityNotAllowedMethodError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityNotAllowedMethodError.php',
    'Twig_Sandbox_SecurityNotAllowedPropertyError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityNotAllowedPropertyError.php',
    'Twig_Sandbox_SecurityNotAllowedTagError' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityNotAllowedTagError.php',
    'Twig_Sandbox_SecurityPolicy' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityPolicy.php',
    'Twig_Sandbox_SecurityPolicyInterface' => $vendorDir . '/twig/twig/lib/Twig/Sandbox/SecurityPolicyInterface.php',
    'Twig_SimpleFilter' => $vendorDir . '/twig/twig/lib/Twig/SimpleFilter.php',
    'Twig_SimpleFunction' => $vendorDir . '/twig/twig/lib/Twig/SimpleFunction.php',
    'Twig_SimpleTest' => $vendorDir . '/twig/twig/lib/Twig/SimpleTest.php',
    'Twig_Source' => $vendorDir . '/twig/twig/lib/Twig/Source.php',
    'Twig_SourceContextLoaderInterface' => $vendorDir . '/twig/twig/lib/Twig/SourceContextLoaderInterface.php',
    'Twig_Template' => $vendorDir . '/twig/twig/lib/Twig/Template.php',
    'Twig_TemplateInterface' => $vendorDir . '/twig/twig/lib/Twig/TemplateInterface.php',
    'Twig_TemplateWrapper' => $vendorDir . '/twig/twig/lib/Twig/TemplateWrapper.php',
    'Twig_Test' => $vendorDir . '/twig/twig/lib/Twig/Test.php',
    'Twig_TestCallableInterface' => $vendorDir . '/twig/twig/lib/Twig/TestCallableInterface.php',
    'Twig_TestInterface' => $vendorDir . '/twig/twig/lib/Twig/TestInterface.php',
    'Twig_Test_Function' => $vendorDir . '/twig/twig/lib/Twig/Test/Function.php',
    'Twig_Test_IntegrationTestCase' => $vendorDir . '/twig/twig/lib/Twig/Test/IntegrationTestCase.php',
    'Twig_Test_Method' => $vendorDir . '/twig/twig/lib/Twig/Test/Method.php',
    'Twig_Test_Node' => $vendorDir . '/twig/twig/lib/Twig/Test/Node.php',
    'Twig_Test_NodeTestCase' => $vendorDir . '/twig/twig/lib/Twig/Test/NodeTestCase.php',
    'Twig_Token' => $vendorDir . '/twig/twig/lib/Twig/Token.php',
    'Twig_TokenParser' => $vendorDir . '/twig/twig/lib/Twig/TokenParser.php',
    'Twig_TokenParserBroker' => $vendorDir . '/twig/twig/lib/Twig/TokenParserBroker.php',
    'Twig_TokenParserBrokerInterface' => $vendorDir . '/twig/twig/lib/Twig/TokenParserBrokerInterface.php',
    'Twig_TokenParserInterface' => $vendorDir . '/twig/twig/lib/Twig/TokenParserInterface.php',
    'Twig_TokenParser_AutoEscape' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/AutoEscape.php',
    'Twig_TokenParser_Block' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Block.php',
    'Twig_TokenParser_Do' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Do.php',
    'Twig_TokenParser_Embed' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Embed.php',
    'Twig_TokenParser_Extends' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Extends.php',
    'Twig_TokenParser_Filter' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Filter.php',
    'Twig_TokenParser_Flush' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Flush.php',
    'Twig_TokenParser_For' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/For.php',
    'Twig_TokenParser_From' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/From.php',
    'Twig_TokenParser_If' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/If.php',
    'Twig_TokenParser_Import' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Import.php',
    'Twig_TokenParser_Include' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Include.php',
    'Twig_TokenParser_Macro' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Macro.php',
    'Twig_TokenParser_Sandbox' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Sandbox.php',
    'Twig_TokenParser_Set' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Set.php',
    'Twig_TokenParser_Spaceless' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Spaceless.php',
    'Twig_TokenParser_Use' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/Use.php',
    'Twig_TokenParser_With' => $vendorDir . '/twig/twig/lib/Twig/TokenParser/With.php',
    'Twig_TokenStream' => $vendorDir . '/twig/twig/lib/Twig/TokenStream.php',
    'Twig_Util_DeprecationCollector' => $vendorDir . '/twig/twig/lib/Twig/Util/DeprecationCollector.php',
    'Twig_Util_TemplateDirIterator' => $vendorDir . '/twig/twig/lib/Twig/Util/TemplateDirIterator.php',
    'WhiteHat101\\Crypt\\APR1_MD5' => $vendorDir . '/whitehat101/apr1-md5/src/APR1_MD5.php',
);
