<?php

echo('<h3>' . $this->t('{core:no_state:suggestions}') . '</h3>');
echo('<ul>');
echo('<li>' . $this->t('{core:no_state:suggestion_goback}') . '</li>');
echo('<li>' . $this->t('{core:no_state:suggestion_closebrowser}') . '</li>');
echo('</ul>');

echo('<h3>' . $this->t('{core:no_state:causes}') . '</h3>');
echo('<ul>');
echo('<li>' . $this->t('{core:no_state:cause_backforward}') . '</li>');
echo('<li>' . $this->t('{core:no_state:cause_openbrowser}') . '</li>');
echo('<li>' . $this->t('{core:no_state:cause_nocookie}') . '</li>');
echo('</ul>');

