    <hr>

    <img src="/{{ baseurlpath }}resources/icons/ssplogo-fish-small.png" alt="Small fish logo" style="float: right">
    Copyright © 2007-2016 <a href="http://uninett.no/">UNINETT AS</a>

    <br style="clear: right">
