<?php
/**
 * Class representing fed Constants.
 *
 * @package SimpleSAMLphp
 */
class sspmod_adfs_SAML2_XML_fed_Const
{
    /**
     * The namespace for WS-FED protocol.
     */
    const NS_FED = 'http://docs.oasis-open.org/wsfed/federation/200706';
}
